import grpc

import aiddl_external_grpc_python.generated.sender_pb2_grpc as sender_pb2_grpc
from aiddl_external_grpc_python.converter import Converter


class FunctionClient:
    def __init__(self, host, port, container):
        channel = grpc.insecure_channel(f'{host}:{port}')
        self.stub = sender_pb2_grpc.SenderStub(channel)
        self.converter = Converter(container)

    def __call__(self, x):
        self.converter.pb2aiddl(self.stub.call(self.converter.aiddl2pb(x)))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: container.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import aiddl_pb2 as aiddl__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0f\x63ontainer.proto\x12\x17org.aiddl.external.grpc\x1a\x0b\x61iddl.proto\"W\n\x13\x46unctionCallRequest\x12\x14\n\x0c\x66unction_uri\x18\x01 \x01(\t\x12*\n\x03\x61rg\x18\x02 \x01(\x0b\x32\x1d.org.aiddl.external.grpc.Term\"z\n\x12\x46unctionCallResult\x12\x35\n\x06status\x18\x01 \x01(\x0e\x32%.org.aiddl.external.grpc.ResultStatus\x12-\n\x06result\x18\x02 \x01(\x0b\x32\x1d.org.aiddl.external.grpc.Term*7\n\x0cResultStatus\x12\x0b\n\x07SUCCESS\x10\x00\x12\x0b\n\x07UNKNOWN\x10\x01\x12\r\n\tEXCEPTION\x10\x02\x32v\n\tContainer\x12i\n\x0c\x46unctionCall\x12,.org.aiddl.external.grpc.FunctionCallRequest\x1a+.org.aiddl.external.grpc.FunctionCallResultB\x19\n\x17org.aiddl.external.grpcb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'container_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\027org.aiddl.external.grpc'
  _RESULTSTATUS._serialized_start=270
  _RESULTSTATUS._serialized_end=325
  _FUNCTIONCALLREQUEST._serialized_start=57
  _FUNCTIONCALLREQUEST._serialized_end=144
  _FUNCTIONCALLRESULT._serialized_start=146
  _FUNCTIONCALLRESULT._serialized_end=268
  _CONTAINER._serialized_start=327
  _CONTAINER._serialized_end=445
# @@protoc_insertion_point(module_scope)



import grpc

import aiddl_external_grpc_python.generated.actor_pb2 as actor_pb2
import aiddl_external_grpc_python.generated.actor_pb2_grpc as actor_pb2_grpc
from aiddl_external_grpc_python.converter import Converter
from aiddl_external_grpc_python.generated.actor_pb2 import REJECTED


class ActorClient:
    def __init__(self, host, port, container):
        channel = grpc.insecure_channel(f'{host}:{port}')
        self.stub = actor_pb2_grpc.ActorStub(channel)
        self.converter = Converter(container)

    def is_supported(self, action):
        answer = self.stub.IsSupported(self.converter.aiddl2pb(action))
        return answer.is_supported

    def dispatch(self, action):
        status = self.stub.Dispatch(self.converter.aiddl2pb(action))
        if status == REJECTED:
            return None
        else:
            return status.id

    def get_status(self, action_id):
        request = actor_pb2.Id()
        request.id = action_id
        status = self.stub.GetStatus(request)
        return status.state, self.converter.pb2aiddl(status.feedback), status.msg

    def cancel(self, action_id):
        request = actor_pb2.Id()
        request.id = action_id
        status = self.stub.Cancel(request)
        return status.state, self.converter.pb2aiddl(status.feedback), status.msg





from enum import Enum
import grpc

import aiddl_external_grpc_python.generated.receiver_pb2 as receiver_pb2
from aiddl_external_grpc_python.converter import Converter


class Order(Enum):
    OLDEST_FIRST = 1
    NEWEST_FIRST = 2


class ReceiverClient:
    def __init__(self, host, port, container,
                 pull_order=Order.OLDEST_FIRST,
                 sort_by=Order.OLDEST_FIRST,
                 pull_max=1,
                 flush_queue=False):
        self.query = None
        channel = grpc.insecure_channel(f'{host}:{port}')
        self.stub = receiver_pb2.SenderStub(channel)
        self.converter = Converter(container)
        self.configureQuery(sort_by=sort_by,
                            pull_order=pull_order,
                            pull_max=pull_max,
                            flush_queue=flush_queue)

    def configure_query(self,
                        sort_by=Order.OLDEST_FIRST,
                        pull_order=Order.OLDEST_FIRST,
                        pull_max=1,
                        flush_queue=False):
        self.query = receiver_pb2.Query()
        if sort_by == Order.OLDEST_FIRST:
            self.query.sort_by = receiver_pb2.OLDEST_FIRST
        else:
            self.query.sort_by = receiver_pb2.NEWEST_FIRST
        if pull_order == Order.OLDEST_FIRST:
            self.query.pull_order = receiver_pb2.OLDEST_FIRST
        else:
            self.query.pull_order = receiver_pb2.NEWEST_FIRST
        self.query.pull_max = pull_max
        self.query.flush_queue = flush_queue

    def Receive(self):
        answer = self.stub.receive(self.query)
        return [self.converter.pb2aiddl(m) for m in answer.messages]



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import actor_pb2 as actor__pb2
from . import aiddl_pb2 as aiddl__pb2


class ActorStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.IsSupported = channel.unary_unary(
                '/org.aiddl.external.grpc.Actor/IsSupported',
                request_serializer=aiddl__pb2.Term.SerializeToString,
                response_deserializer=actor__pb2.Supported.FromString,
                )
        self.Dispatch = channel.unary_unary(
                '/org.aiddl.external.grpc.Actor/Dispatch',
                request_serializer=aiddl__pb2.Term.SerializeToString,
                response_deserializer=actor__pb2.Status.FromString,
                )
        self.GetStatus = channel.unary_unary(
                '/org.aiddl.external.grpc.Actor/GetStatus',
                request_serializer=actor__pb2.Id.SerializeToString,
                response_deserializer=actor__pb2.Status.FromString,
                )
        self.Cancel = channel.unary_unary(
                '/org.aiddl.external.grpc.Actor/Cancel',
                request_serializer=actor__pb2.Id.SerializeToString,
                response_deserializer=actor__pb2.Status.FromString,
                )


class ActorServicer(object):
    """Missing associated documentation comment in .proto file."""

    def IsSupported(self, request, context):
        """Ask actor if an AIDDL string is is considered as a supported action
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Dispatch(self, request, context):
        """Request action. Returns status with Id that can be used to request state updates or cancel execution.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetStatus(self, request, context):
        """Get current status of an action based on its id.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Cancel(self, request, context):
        """Cancel an action based on its id. Also returns status after cancelation request.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ActorServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'IsSupported': grpc.unary_unary_rpc_method_handler(
                    servicer.IsSupported,
                    request_deserializer=aiddl__pb2.Term.FromString,
                    response_serializer=actor__pb2.Supported.SerializeToString,
            ),
            'Dispatch': grpc.unary_unary_rpc_method_handler(
                    servicer.Dispatch,
                    request_deserializer=aiddl__pb2.Term.FromString,
                    response_serializer=actor__pb2.Status.SerializeToString,
            ),
            'GetStatus': grpc.unary_unary_rpc_method_handler(
                    servicer.GetStatus,
                    request_deserializer=actor__pb2.Id.FromString,
                    response_serializer=actor__pb2.Status.SerializeToString,
            ),
            'Cancel': grpc.unary_unary_rpc_method_handler(
                    servicer.Cancel,
                    request_deserializer=actor__pb2.Id.FromString,
                    response_serializer=actor__pb2.Status.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'org.aiddl.external.grpc.Actor', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Actor(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def IsSupported(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/org.aiddl.external.grpc.Actor/IsSupported',
            aiddl__pb2.Term.SerializeToString,
            actor__pb2.Supported.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Dispatch(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/org.aiddl.external.grpc.Actor/Dispatch',
            aiddl__pb2.Term.SerializeToString,
            actor__pb2.Status.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetStatus(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/org.aiddl.external.grpc.Actor/GetStatus',
            actor__pb2.Id.SerializeToString,
            actor__pb2.Status.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Cancel(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/org.aiddl.external.grpc.Actor/Cancel',
            actor__pb2.Id.SerializeToString,
            actor__pb2.Status.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: actor.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import aiddl_pb2 as aiddl__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0b\x61\x63tor.proto\x12\x17org.aiddl.external.grpc\x1a\x0b\x61iddl.proto\"\x81\x01\n\x06Status\x12\n\n\x02id\x18\x01 \x01(\x04\x12-\n\x05state\x18\x02 \x01(\x0e\x32\x1e.org.aiddl.external.grpc.State\x12/\n\x08\x66\x65\x65\x64\x62\x61\x63k\x18\x03 \x01(\x0b\x32\x1d.org.aiddl.external.grpc.Term\x12\x0b\n\x03msg\x18\x04 \x01(\t\"\x10\n\x02Id\x12\n\n\x02id\x18\x01 \x01(\x04\"!\n\tSupported\x12\x14\n\x0cis_supported\x18\x01 \x01(\x08*\x84\x01\n\x05State\x12\x0b\n\x07PENDING\x10\x00\x12\n\n\x06\x41\x43TIVE\x10\x01\x12\r\n\tSUCCEEDED\x10\x02\x12\t\n\x05\x45RROR\x10\x03\x12\r\n\tRECALLING\x10\x04\x12\x0c\n\x08RECALLED\x10\x05\x12\x0e\n\nPREEMPTING\x10\x06\x12\r\n\tPREEMPTED\x10\x07\x12\x0c\n\x08REJECTED\x10\x08\x32\xb8\x02\n\x05\x41\x63tor\x12P\n\x0bIsSupported\x12\x1d.org.aiddl.external.grpc.Term\x1a\".org.aiddl.external.grpc.Supported\x12J\n\x08\x44ispatch\x12\x1d.org.aiddl.external.grpc.Term\x1a\x1f.org.aiddl.external.grpc.Status\x12I\n\tGetStatus\x12\x1b.org.aiddl.external.grpc.Id\x1a\x1f.org.aiddl.external.grpc.Status\x12\x46\n\x06\x43\x61ncel\x12\x1b.org.aiddl.external.grpc.Id\x1a\x1f.org.aiddl.external.grpc.StatusB\x19\n\x17org.aiddl.external.grpcb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'actor_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\027org.aiddl.external.grpc'
  _STATE._serialized_start=239
  _STATE._serialized_end=371
  _STATUS._serialized_start=54
  _STATUS._serialized_end=183
  _ID._serialized_start=185
  _ID._serialized_end=201
  _SUPPORTED._serialized_start=203
  _SUPPORTED._serialized_end=236
  _ACTOR._serialized_start=374
  _ACTOR._serialized_end=686
# @@protoc_insertion_point(module_scope)
